@@ -3,7 +3,7 @@
 //! Print some information about a Blender scene file.
 //! ```shell
 //! $ ./target/release/blend_info -h
-//! blend_info 0.2.5
+//! blend_info 0.2.7
 //! Print some information about a Blender scene file
 //!
 //! USAGE:
@@ -22,7 +22,45 @@
 //!     <path>    The path to the file to read
 //! ```
 //!
-//! ## Examples
+//! ## Usage as a crate (used in your own code)
+//!
+//! ```rust
+//! use blend_info::{read_dna, DnaStrC};
+//! // std
+//! use std::collections::HashMap;
+//! use std::path::PathBuf;
+//!
+//! fn main() -> std::io::Result<()> {
+//!     println!("read_dna");
+//!     // .blend file
+//!     let path: PathBuf = PathBuf::from(r"blend/factory_v279.blend");
+//!     println!("Try to read {:?} file ...", path);
+//!     // read DNA
+//!     let mut dna_types_hm: HashMap<String, u16> = HashMap::new();
+//!     let mut dna_structs_hm: HashMap<String, DnaStrC> = HashMap::new();
+//!     let mut dna_pointers_hm: HashMap<usize, usize> = HashMap::new();
+//!     let mut dna_2_type_id: Vec<u16> = Vec::new();
+//!     let mut types: Vec<String> = Vec::new();
+//!     let mut num_bytes_read: usize = 0;
+//!     let print_dna: bool = false;
+//!     let print_pointers: bool = false;
+//!     read_dna(
+//!         print_dna,
+//!         print_pointers,
+//!         &path,
+//!         &mut dna_types_hm,
+//!         &mut dna_structs_hm,
+//!         &mut dna_pointers_hm,
+//!         &mut dna_2_type_id,
+//!         &mut types,
+//!         &mut num_bytes_read,
+//!     )?;
+//!     println!("{} {:?}", num_bytes_read, &path);
+//!     Ok(())
+//! }
+//! ```
+//!
+//! ## Examples (used as a standalone executable to query .blend files)
 //!
 //! ### DNA
 //!
@@ -613,6 +651,22 @@ pub fn get_float3(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -
     float_values
 }
 
+pub fn get_float4(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> [f32; 4] {
+    let mut float_values: [f32; 4] = [0.0; 4];
+    if member.mem_type.as_str() == "float" {
+        for i in 0..4 {
+            let mut float_buf: [u8; 4] = [0_u8; 4];
+            for b in 0..4 as usize {
+                float_buf[b] = bytes_read[byte_index + i * 4 + b];
+            }
+            float_values[i] = unsafe { mem::transmute(float_buf) };
+        }
+    } else {
+        println!("WARNING: \"float\" expected, {:?} found", member.mem_type);
+    }
+    float_values
+}
+
 pub fn get_int(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> i32 {
     let mut int_value: i32 = 0;
     if member.mem_type.as_str() == "int" {
@@ -701,6 +755,25 @@ pub fn print_pointer(
     }
 }
 
+/// Read a `.blend` file to extract DNA information first
+///
+/// Verbosity flags:
+///
+/// * `print_dna`
+/// * `print_pointers`
+///
+/// Input
+///
+/// * `path` - a path to a folder/filename.blend
+///
+/// Return values (`&mut`):
+/// * `dna_types_hm` - A HashMap with a type name (e.g. "float") and it's byte size as u16
+/// * `dna_structs_hm` - A HashMap with a struct name (e.g. "Camera") and it's members (DnaStrC)
+/// * `dna_pointers_hm` - A HashMap with 2 usize values (real memory
+/// address stored in file, and byte position within .blend file)
+/// * `dna_2_type_id` - Use `sdna_nr` to find `type_id`
+/// * `types` - Vec of type names (e.g. "char", "short", "double")
+/// * `bytes_read` - Should match the file size on disk
 pub fn read_dna(
     print_dna: bool,
     print_pointers: bool,
@@ -1173,7 +1246,7 @@ pub fn use_dna(
                         bytes_read.append(&mut buffer);
                         structs_read.push(name.clone());
                         data_read.push(len);
-			pointers_read.push((old, sdna_nr));
+                        pointers_read.push((old, sdna_nr));
                     }
                 }
             } else {
@@ -1195,7 +1268,7 @@ pub fn use_dna(
                         bytes_read.append(&mut buffer);
                         structs_read.push(name.clone());
                         data_read.push(len);
-			pointers_read.push((old, sdna_nr));
+                        pointers_read.push((old, sdna_nr));
                     }
                 }
             }
